# File : Makefile.PL
use ExtUtils::MakeMaker;
use Config;
#$ExtUtils::MakeMaker::Verbose=2;

require 5.000;

my $modname="PBS";

my $PBS_INCLUDE;
my $PBS_LIB;
my $DHAVE_RPP_H="";
my $DHAVE_RM_H="";
my $BROKEN_PBSE_TO_TXT="";
my $have_curses;

my $header_mismatch; # set to 1 if PBS_wrap.c must be regenerated

my $cpan2rpm="./cpan2rpm";
my $rpmbuild="rpmbuild";
my $swig="swig";

print "Configuring perl-PBS... ";

open V, "VERSION" or die "Can't open VERSION... time to kill myself.\n";
my $version=<V>;
chomp $version;
close V;
print "version $version\n";


print "Checking for pbs_ifl.h... ";
foreach my $dir (split(" ", "$ENV{PBS_INCLUDE} $ENV{PBS_HOME}/include /usr/local/torque/include /usr/local/pbs/include /opt/torque/include /usr/pbs/include /usr/local/include /usr/include")) {
   if (-f "$dir/pbs_ifl.h") {
      $PBS_INCLUDE=$dir;
      last;
   }
}
if (-f "$PBS_INCLUDE/pbs_ifl.h") {
   print "yes  $PBS_INCLUDE/pbs_ifl.h\n";
} else {
   print "no\n*** Error: set PBS_INCLUDE to the directory with pbs_ifl.h\n";
   exit(1);
}

print "Checking for rpp.h... ";
if (-f "$PBS_INCLUDE/rpp.h") {
   print "yes\n";
   $DHAVE_RPP_H="-DHAVE_RPP_H";
} else {
   print "no\n    -> Using an internal copy, cross your fingers!\n";
}

print "Checking for rm.h... ";
if (-f "$PBS_INCLUDE/rm.h") {
   print "yes\n";
   $DHAVE_RM_H="-DHAVE_RM_H";
} else {
   print "no\n    -> Using an internal copy, cross your fingers!\n";
}

print "Checking if pbs_selstat uses struct attrib... ";
open(H, "$PBS_INCLUDE/pbs_ifl.h") or die;
($selstat)=grep /pbs_selstat/, (<H>);
close H;
chomp($selstat);
if (!$selstat) {
   print "Unable to find pbs_selstat() declaration in pbs_ifl.h!\n";
   exit(1);
}
$selstat =~ s/.*,.*,([^)]+).*/$1/;
if ($selstat =~ /attrib/) {
   $header_mismatch=1;
   print "yes  (Hrm, must be PBSPro)\n";
} else {
   $selstat="";
   print "no\n";
}

print "Checking for broken pbse_to_txt... ";
open(H, "$PBS_INCLUDE/pbs_error.h") or die "Unable to open $PBS_INCLUDE/pbs_error.h";
($pbse)=grep /pbse_to_txt\(\)/, (<H>);
close H;
chomp($pbse);
if ($pbse) {
   $BROKEN_PBSE_TO_TXT="-DBROKEN_PBSE_TO_TXT";
   print "yes\n";
} else {
   print "no\n";
}

print "Checking for number of args in rpp_open... ";
my $sys_numargs=0;
my $wrap_numargs=0;
my @tmp=();
if ($DHAVE_RPP_H) {
   open(H,"$PBS_INCLUDE/rpp.h") or die "Unable to open $PBS_INCLUDE/rpp.h";
} else {
   open(H,"PBS.i") or die "Unable to open PBS.i";
}
($rpp_open)=grep /rpp_open/, (<H>);
close H;
$rpp_open =~ s/.*\((.*)\).*/$1/; # get the arg list
@tmp=split(',', $rpp_open);
if (scalar @tmp >= 2) {
    $sys_numargs=scalar @tmp;
    print "$sys_numargs\n";
} else {
    die "$sys_numargs, which doesn't make sense, aborting";
}

print "Checking for rpp_open args in PBS_wrap.c... ";
if (-f "PBS_wrap.c") {
   open(H,"PBS_wrap.c") or die "Unable to open PBS_wrap.c";
   ($rpp_open)=grep /rpp_open.*arg1,arg2/, (<H>);
   close H;
   $rpp_open =~ s/.*rpp_open.*\((.*)\).*/$1/; # get the arg list
   @tmp=split(',', $rpp_open);
   $wrap_numargs=scalar @tmp;
   print "$wrap_numargs\n";
   if ($sys_numargs != $wrap_numargs) {
      $header_mismatch=1;
   }
} else {
   print "unknown\n";
   $header_mismatch=1;
}

print "Checking for libpbs.a... ";
foreach my $dir (split(" ", "$ENV{PBS_LIB} $ENV{PBS_HOME}/lib $ENV{PBS_HOME}/lib64 /usr/local/torque/lib /usr/local/torque/lib64 /usr/local/pbs/lib /usr/local/pbs/lib64 /usr/lib/torque /usr/lib64/torque /usr/lib/torque/lib /usr/lib64/torque/lib /usr/lib/torque/lib64 /usr/lib64/torque/lib64 /opt/torque/lib /opt/torque/lib64 /usr/pbs/lib /usr/pbs/lib64 /usr/local/lib /usr/local/lib64 /usr/lib /usr/lib64")) {
   if (-f "$dir/libpbs.a") {
      $PBS_LIB=$dir;
      last;
   }
}
if (-f "$PBS_LIB/libpbs.a") {
   print "yes  $PBS_LIB/libpbs.a\n";
} else {
   print "no\n*** Error: set PBS_LIB to the directory with libpbs.a\n";
   exit(1);
}

print "Checking for Curses... ";
eval 'use Curses';
if ($@) {
   print "no\n    -> pbstop will be installed, but unusable until Curses is installed.\n";
} else {
   $have_curses="yes";
   print "yes\n";
}

print "Checking if we need to regenerate PBS_wrap.c... ";
if ($header_mismatch) {
   print "yes\n";

   print "Checking for swig >= 1.3.19... ";
   $swigversion=`$swig -version 2>&1 | grep Version`;
   chomp($swigversion);
   $swigversion =~ s/SWIG Version //;
   ($svM, $svm, $svr)=split(/\./, $swigversion);
   if (!($svM >= 1 and $svm >= 3 and $svr >= 19)) {
      print "no\n    -> Error: we need swig >= 1.3.19 to continue.\n";
      exit(1);
   } else {
      print "yes\n";
      utime undef, undef, "PBS_wrap.c";
   }

} else {
   print "no\n";
}
   

print "Configuration for perl-PBS: -L$PBS_LIB -I$PBS_INCLUDE $DHAVE_RPP_H $DHAVE_RM_H $BROKEN_PBSE_TO_TXT\n";


WriteMakefile(
	NAME		=>  $modname,
	DISTNAME	=>  "perl-$modname",
        ($] >= 5.005) ?
	(AUTHOR		=>  "Garrick Staples <garrick\@usc.edu>") : (),
	VERSION		=>  "$version",
	LIBS		=> ["-L$PBS_LIB  -lnet -lcmds -lpbs -llog"],
	INC		=>  "-I$PBS_INCLUDE $DHAVE_RPP_H $DHAVE_RM_H $BROKEN_PBSE_TO_TXT",
	OBJECT		=>  "${modname}_wrap.o",
	EXE_FILES	=> ["bin/pbstop"],
	ABSTRACT_FROM	=> "src/PBS.pm",
	macro		=> {CPAN2RPM => $cpan2rpm, RPMBUILD=>$rpmbuild, SWIG=>$swig, have_curses=>$have_curses},
	clean		=> {FILES => "bin/pbstop perl-${modname}.spec"},
);

sub MY::postamble {
    return <<"MAKE_FRAG";

# core and pm stuff
Makefile: VERSION
${modname}_wrap.c: ${modname}.i
	\$(SWIG) -perl5 $DHAVE_RPP_H $DHAVE_RM_H -I$PBS_INCLUDE ${modname}.i
	\@\$(RM) -f ${modname}.pm

lib/${modname}.pm: VERSION src/${modname}.pm
	\$(PERL) -p -e 's{\\%VERSION%}{'"$version"'}' < src/${modname}.pm > lib/${modname}.pm


# pbstop
# if installing into a non-standard location, be sure to add a path to the installed
# perl-PBS module
pbstop: bin/pbstop
bin/pbstop: Makefile src/pbstop
	\@if [ "\$(PERL_ARCHLIB)" = "\$(INSTALLARCHLIB)" ];then \\
	    \$(PERL) -p -e 's{.*INSTALLARCHLIB.*}{}' < src/pbstop > bin/pbstop; \\
	else \\
	    echo "\$(PERL) -p -e 's{INSTALLARCHLIB}{\$(INSTALLARCHLIB)}' < src/pbstop > bin/pbstop"; \\
	    \$(PERL) -p -e 's{INSTALLARCHLIB}{\$(INSTALLARCHLIB)}' < src/pbstop > bin/pbstop; \\
	fi
	\$(CHMOD) 755 bin/pbstop
	\@if [ -n "\$(have_curses)" ];then \\
		echo "\$(PERL) -c bin/pbstop"; \\
		\$(PERL) -c bin/pbstop; \\
	fi
	


# rpm and spec targets
spec: perl-${modname}.spec
perl-${modname}.spec: Makefile.PL VERSION  CHANGELOG
	\$(CPAN2RPM)  --name=perl-${modname} --no-prfx \\
	    --spec=`pwd`/perl-${modname}.spec --spec-only \\
	    --doc "=README LICENSE demos/pbsnodes-a demos/qstat-f demos/dumpmom" \\
	    --prologue="build:PBS_INCLUDE=\"$PBS_INCLUDE\"; PBS_LIB=\"$PBS_LIB\"; export PBS_INCLUDE PBS_LIB" \\
	    --epilogue="changelog:file:CHANGELOG" --no-default-sec=changelog .

rpm: spec dist
	\$(CP) -fp \$(DISTVNAME).tar\$(SUFFIX) `rpm --eval '%_sourcedir'`
	\$(RPMBUILD) -ba perl-${modname}.spec

srpm: spec dist
	\$(CP) -fp \$(DISTVNAME).tar\$(SUFFIX) `rpm --eval '%_sourcedir'`
	\$(RPMBUILD) -bs perl-${modname}.spec

MAKE_FRAG
}

sub ExtUtils::MM_Unix::parse_abstract {
   return "Perl extension for PBS";
}
